Imports System.Threading
Imports Syncfusion.Windows.Forms.Chart
Imports PLCDemo.Chart

Namespace Forms

    Public Class frmMain

#Region " structures & enums "
        Public Structure PlotPoints
            Public xPoints As ArrayList
            Public yPoints As ArrayList

            Public Sub New(ByVal xPoints As ArrayList, ByVal yPoints As ArrayList)
                Me.xPoints = xPoints
                Me.yPoints = yPoints
            End Sub
        End Structure
#End Region

#Region " declarations "
        Private Const WAIT_BETWEEN_RECORDINGS As Integer = 3000
        Private Const UPDATE_INTERVALL As Integer = 300
        Private Const SPS_ARRAY_SIZE As Integer = 2000

        Private arrayListModel As ArraylistModel
        Private reloadThread As Thread
        Private dataSource As List(Of PlotPoints)
        Private invokeErrorDelegate As ErrorDelegate
        Private cancelReload As Boolean

        Private Delegate Sub ErrorDelegate(ByVal message As String)
#End Region

#Region " constructor / destructor "
        Public Sub New()


            InitializeComponent()


            Me.arrayListModel = New ArraylistModel(Me)
            Me.InitializeControlSettings()
        End Sub
#End Region

#Region " events "
        Private Sub frmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
            Me.StartRecoding()
        End Sub
#End Region

#Region " methods "
        Private Sub InitializeControlSettings()
            Dim series As ChartSeries = Me.ccPLCData.Model.NewSeries("PLC-Data")

            Me.ccPLCData.Palette = ChartColorPalette.Analog
            Me.ccPLCData.Text = "Recording"
            Me.ccPLCData.Font = New System.Drawing.Font("Arial", 12.0F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point)
            Me.ccPLCData.ChartInterior = New Syncfusion.Drawing.BrushInfo(New Syncfusion.Drawing.BrushInfo(Syncfusion.Drawing.GradientStyle.None, Color.Black, Color.White))
            Me.ccPLCData.ChartArea.HidePartialLabels = True
            Me.ccPLCData.Indexed = False
            Me.ccPLCData.ChartAreaShadow = False
            series.Text = series.Name
            series.Type = Syncfusion.Windows.Forms.Chart.ChartSeriesType.Column

            series.Style.DisplayShadow = False
            series.Style.DisplayText = True
            series.SeriesModelImpl = Me.arrayListModel
            Me.ccPLCData.Series.Add(series)
            Me.ccPLCData.PrimaryXAxis.Title = "X"
            Me.ccPLCData.PrimaryYAxis.Title = "Y"
            Me.ccPLCData.PrimaryXAxis.RangeType = ChartAxisRangeType.Set
            Me.ccPLCData.PrimaryYAxis.RangeType = ChartAxisRangeType.Set
            Me.ccPLCData.PrimaryXAxis.RangePaddingType = ChartAxisRangePaddingType.None
            Me.ccPLCData.PrimaryYAxis.RangePaddingType = ChartAxisRangePaddingType.None
            Me.ccPLCData.PrimaryXAxis.Range = New MinMaxInfo(0, 2000, 250)
            Me.ccPLCData.PrimaryYAxis.Range = New MinMaxInfo(0, 9000, 1000)

            Me.ccPLCData.Indexed = False
            Me.ccPLCData.CalcRegions = False
            series.EnableStyles = False
            series.Style.DisplayShadow = False
            Me.ccPLCData.DropSeriesPoints = True
        End Sub

        Public Function StartRecoding() As Boolean
            Try
                Me.StopReloadThread()
                Me.arrayListModel.Clear()
                Me.cancelReload = False
                Me.reloadThread = New Thread(AddressOf ReloadData)
                Me.reloadThread.Name = "Reload thread"
                Me.reloadThread.Start()
                Return True
            Catch ex As Exception
                Return False
            End Try
        End Function

        Public Function StopRecording() As Boolean
            Try
                Me.cancelReload = True
                Me.StopReloadThread()
                Return True
            Catch ex As Exception
                Return False
            End Try
        End Function

        Private Sub StopReloadThread()
            If Not Me.reloadThread Is Nothing Then
                If Not Me.reloadThread.Join(3000) Then
                    Me.reloadThread.Abort()
                End If
                Me.reloadThread = Nothing
            End If
        End Sub

        Private Sub ReloadData()
            Dim start As DateTime

            Try
                While Not Me.cancelReload AndAlso Not Me.Disposing AndAlso Not Me.IsDisposed
                    Me.FillPoints()

                    For i As Integer = 0 To Me.dataSource.Count - 1
                        If Me.cancelReload OrElse Me.Disposing Then
                            Exit Sub
                        End If

                        Me.arrayListModel.AddRange(Me.dataSource.Item(i).xPoints, Me.dataSource.Item(i).yPoints)
                        Thread.Sleep(UPDATE_INTERVALL)
                    Next

                    start = Now
                    Do
                        If Me.cancelReload OrElse Me.Disposing Then
                            Exit Sub
                        End If
                        Thread.Sleep(20)
                    Loop Until Now.Subtract(start).TotalMilliseconds > WAIT_BETWEEN_RECORDINGS

                    Me.arrayListModel.Clear()
                End While
            Catch ex As Exception
                Me.InvokeErrorOccured(ex.Message)
            End Try
        End Sub

        Private Sub FillPoints()
            Dim yPoint As Integer = 0
            Dim rand As New Random
            Dim lastNumberOfReadData As Short = 0
            Dim tempXPoints As New ArrayList
            Dim tempYPoints As New ArrayList

            If Me.dataSource Is Nothing Then
                Me.dataSource = New List(Of PlotPoints)
            Else
                Me.dataSource.Clear()
            End If

            For i As Integer = 1 To 16
                tempXPoints.Clear()
                tempYPoints.Clear()
                For j As Integer = lastNumberOfReadData To (i * (SPS_ARRAY_SIZE / 16)) - 1
                    tempXPoints.Add(Convert.ToDouble(j))
                    tempYPoints.Add(New Double() {Convert.ToDouble(yPoint)})
                    yPoint += rand.Next(1, 8)
                Next
                Me.dataSource.Add(New PlotPoints(tempXPoints.Clone, tempYPoints.Clone))
                lastNumberOfReadData = i * 125
            Next
        End Sub

        Private Sub InvokeErrorOccured(ByVal message As String)
            If Me.IsHandleCreated Then
                If Me.invokeErrorDelegate Is Nothing Then
                    Me.invokeErrorDelegate = New ErrorDelegate(AddressOf ErrorOccured)
                End If
                Me.Invoke(Me.invokeErrorDelegate, New Object() {message})
            End If
        End Sub

        Private Sub ErrorOccured(ByVal message As String)
            Me.StopRecording()
            MsgBox(message, MsgBoxStyle.Critical)
        End Sub
#End Region

    End Class

End Namespace